@@ -7,8 +7,8 @@ import fnmatch
 import os
 import posixpath
 import re
+import subprocess  # nosec: Needed for gitignore support.
 import sys
-from distutils.util import strtobool as _as_bool
 from functools import lru_cache
 from pathlib import Path
 from typing import Any, Callable, Dict, FrozenSet, Iterable, List, Optional, Pattern, Set, Tuple
@@ -17,10 +17,10 @@ from warnings import warn
 from . import stdlibs
 from ._future import dataclass, field
 from ._vendored import toml
-from .exceptions import ProfileDoesNotExist
+from .exceptions import FormattingPluginDoesNotExist, InvalidSettingsPath, ProfileDoesNotExist
 from .profiles import profiles
 from .sections import DEFAULT as SECTION_DEFAULTS
-from .sections import FIRSTPARTY, FUTURE, STDLIB, THIRDPARTY
+from .sections import FIRSTPARTY, FUTURE, LOCALFOLDER, STDLIB, THIRDPARTY
 from .wrap_modes import WrapModes
 from .wrap_modes import from_string as wrap_mode_from_string
 
@@ -58,10 +58,28 @@ KNOWN_SECTION_MAPPING: Dict[str, str] = {
     FUTURE: "FUTURE_LIBRARY",
     FIRSTPARTY: "FIRST_PARTY",
     THIRDPARTY: "THIRD_PARTY",
+    LOCALFOLDER: "LOCAL_FOLDER",
 }
 
 RUNTIME_SOURCE = "runtime"
 
+DEPRECATED_SETTINGS = ("not_skip", "keep_direct_and_as_imports")
+
+_STR_BOOLEAN_MAPPING = {
+    "y": True,
+    "yes": True,
+    "t": True,
+    "on": True,
+    "1": True,
+    "true": True,
+    "n": False,
+    "no": False,
+    "f": False,
+    "off": False,
+    "0": False,
+    "false": False,
+}
+
 
 @dataclass(frozen=True)
 class _Config:
@@ -92,6 +110,7 @@ class _Config:
         }
     )
     skip_glob: FrozenSet[str] = frozenset()
+    skip_gitignore: bool = False
     line_length: int = 79
     wrap_length: int = 0
     line_ending: str = ""
@@ -100,6 +119,7 @@ class _Config:
     known_future_library: FrozenSet[str] = frozenset(("__future__",))
     known_third_party: FrozenSet[str] = frozenset(("google.appengine.api",))
     known_first_party: FrozenSet[str] = frozenset()
+    known_local_folder: FrozenSet[str] = frozenset()
     known_standard_library: FrozenSet[str] = frozenset()
     extra_standard_library: FrozenSet[str] = frozenset()
     known_other: Dict[str, FrozenSet[str]] = field(default_factory=dict)
@@ -111,6 +131,7 @@ class _Config:
     length_sort_sections: FrozenSet[str] = frozenset()
     add_imports: FrozenSet[str] = frozenset()
     remove_imports: FrozenSet[str] = frozenset()
+    append_only: bool = False
     reverse_relative: bool = False
     force_single_line: bool = False
     single_line_exclusions: Tuple[str, ...] = ()
@@ -125,7 +146,6 @@ class _Config:
     lines_between_types: int = 0
     combine_as_imports: bool = False
     combine_star: bool = False
-    keep_direct_and_as_imports: bool = True
     include_trailing_comma: bool = False
     from_first: bool = False
     verbose: bool = False
@@ -147,8 +167,14 @@ class _Config:
     ensure_newline_before_comments: bool = False
     directory: str = ""
     profile: str = ""
+    honor_noqa: bool = False
     src_paths: FrozenSet[Path] = frozenset()
     old_finders: bool = False
+    remove_redundant_aliases: bool = False
+    float_to_top: bool = False
+    filter_files: bool = False
+    formatter: str = ""
+    formatting_function: Optional[Callable[[str, str, object], str]] = None
 
     def __post_init__(self):
         py_version = self.py_version
@@ -203,11 +229,15 @@ class Config(_Config):
         config: Optional[_Config] = None,
         **config_overrides,
     ):
+        self._known_patterns: Optional[List[Tuple[Pattern[str], str]]] = None
+        self._section_comments: Optional[Tuple[str, ...]] = None
+
         if config:
             config_vars = vars(config).copy()
             config_vars.update(config_overrides)
             config_vars["py_version"] = config_vars["py_version"].replace("py", "")
             config_vars.pop("_known_patterns")
+            config_vars.pop("_section_comments")
             super().__init__(**config_vars)  # type: ignore
             return
 
@@ -222,6 +252,10 @@ class Config(_Config):
             )
             project_root = os.path.dirname(settings_file)
         elif settings_path:
+            if not os.path.exists(settings_path):
+                raise InvalidSettingsPath(settings_path)
+
+            settings_path = os.path.abspath(settings_path)
             project_root, config_settings = _find_config(settings_path)
         else:
             config_settings = {}
@@ -230,6 +264,12 @@ class Config(_Config):
         profile_name = config_overrides.get("profile", config_settings.get("profile", ""))
         profile: Dict[str, Any] = {}
         if profile_name:
+            if profile_name not in profiles:
+                import pkg_resources
+
+                for plugin in pkg_resources.iter_entry_points("isort.profiles"):
+                    profiles.setdefault(plugin.name, plugin.load())
+
             if profile_name not in profiles:
                 raise ProfileDoesNotExist(profile_name)
 
@@ -263,8 +303,16 @@ class Config(_Config):
                 "known_future_library",
                 "known_third_party",
                 "known_first_party",
+                "known_local_folder",
             ):
-                known_other[key[len(KNOWN_PREFIX) :].lower()] = frozenset(value)
+                import_heading = key[len(KNOWN_PREFIX) :].lower()
+                known_other[import_heading] = frozenset(value)
+                if not import_heading.upper() in combined_config.get("sections", ()):
+                    warn(
+                        f"`{key}` setting is defined, but not {import_heading.upper()} is not"
+                        " included in `sections` config option:"
+                        f" {combined_config.get('sections', SECTION_DEFAULTS)}."
+                    )
             if key.startswith(IMPORT_HEADING_PREFIX):
                 import_headings[key[len(IMPORT_HEADING_PREFIX) :].lower()] = str(value)
 
@@ -275,6 +323,17 @@ class Config(_Config):
 
             combined_config[key] = type(default_value)(value)
 
+        for section in combined_config.get("sections", ()):
+            if section in SECTION_DEFAULTS:
+                continue
+            elif not section.lower() in known_other:
+                config_keys = ", ".join(known_other.keys())
+                warn(
+                    f"`sections` setting includes {section}, but no known_{section.lower()} "
+                    "is defined. "
+                    f"The following known_SECTION config options are defined: {config_keys}."
+                )
+
         if "directory" not in combined_config:
             combined_config["directory"] = (
                 os.path.dirname(config_settings["source"])
@@ -291,11 +350,31 @@ class Config(_Config):
                 path_root / path for path in combined_config.get("src_paths", ())
             )
 
+        if "formatter" in combined_config:
+            import pkg_resources
+
+            for plugin in pkg_resources.iter_entry_points("isort.formatters"):
+                if plugin.name == combined_config["formatter"]:
+                    combined_config["formatting_function"] = plugin.load()
+                    break
+            else:
+                raise FormattingPluginDoesNotExist(combined_config["formatter"])
+
         # Remove any config values that are used for creating config object but
         # aren't defined in dataclass
         combined_config.pop("source", None)
         combined_config.pop("sources", None)
         combined_config.pop("runtime_src_paths", None)
+
+        for deprecated_option in DEPRECATED_SETTINGS:
+            if deprecated_option in combined_config:
+                warn(
+                    f"\n\nThe following deprecated settings was used: {deprecated_option}!\n"
+                    "Please see the 5.0.0 upgrade guide:\n"
+                    "\thttps://timothycrosley.github.io/isort/docs/upgrade_guides/5.0.0/\n"
+                )
+                combined_config.pop(deprecated_option)
+
         if known_other:
             for known_key in known_other:
                 combined_config.pop(f"{KNOWN_PREFIX}{known_key}", None)
@@ -305,7 +384,6 @@ class Config(_Config):
                 combined_config.pop(f"{IMPORT_HEADING_PREFIX}{import_heading_key}")
             combined_config["import_headings"] = import_headings
 
-        self._known_patterns: Optional[List[Tuple[Pattern[str], str]]] = None
         super().__init__(sources=tuple(sources), **combined_config)  # type: ignore
 
     def is_skipped(self, file_path: Path) -> bool:
@@ -317,6 +395,13 @@ class Config(_Config):
 
         os_path = str(file_path)
 
+        if self.skip_gitignore:
+            result = subprocess.run(  # nosec
+                ["git", "-C", str(file_path.resolve().parent), "check-ignore", "--quiet", os_path]
+            )
+            if result.returncode == 0:
+                return True
+
         normalized_path = os_path.replace("\\", "/")
         if normalized_path[1:2] == ":":
             normalized_path = normalized_path[2:]
@@ -365,6 +450,14 @@ class Config(_Config):
 
         return self._known_patterns
 
+    @property
+    def section_comments(self) -> Tuple[str, ...]:
+        if self._section_comments is not None:
+            return self._section_comments
+
+        self._section_comments = tuple(f"# {heading}" for heading in self.import_headings.values())
+        return self._section_comments
+
     @staticmethod
     def _parse_known_pattern(pattern: str) -> List[str]:
         """Expand pattern if identified as a directory and return found sub packages"""
@@ -508,7 +601,7 @@ def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
             elif existing_value_type == bool:
                 # Only some configuration formats support native boolean values.
                 if not isinstance(value, bool):
-                    value = bool(_as_bool(value))
+                    value = _as_bool(value)
                 settings[key] = value
             elif key.startswith(KNOWN_PREFIX):
                 settings[key] = _abspaths(os.path.dirname(file_path), _as_list(value))
@@ -526,4 +619,14 @@ def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
     return settings
 
 
+def _as_bool(value: str) -> bool:
+    """Given a string value that represents True or False, returns the Boolean equivalent.
+    Heavily inspired from distutils strtobool.
+    """
+    try:
+        return _STR_BOOLEAN_MAPPING[value.lower()]
+    except KeyError:
+        raise ValueError(f"invalid truth value {value}")
+
+
 DEFAULT_CONFIG = Config()
